package org.act.od.impl.view {
  import mx.containers.Box;
  import mx.containers.HDividedBox;
  import mx.containers.TabNavigator;
  import mx.containers.VDividedBox;
  import mx.controls.Button;
  import mx.core.ScrollPolicy;
  import mx.effects.WipeRight;
  import mx.events.FlexEvent;
  import mx.events.PropertyChangeEvent;
  import mx.events.PropertyChangeEventKind;
  import mx.managers.ToolTipManager;

  import org.act.od.impl.view.editor.Microimage;
  import org.act.od.impl.view.fileNavigator.FileNavigatorTab;
  import org.act.od.impl.view.toolMenuBar.DesignerToolBar;
  import org.ow2.orchestra.designer.model.DesignerModelLocator;
  import org.ow2.orchestra.designer.view.ProcessTabNavigator;
  import org.ow2.orchestra.designer.view.menuBar.DesignerMenuBar;
  import org.ow2.orchestra.designer.view.palette.BpmnPalette;
  import org.ow2.orchestra.designer.view.properties.PropertyView;
  import org.ow2.orchestra.designer.view.wsdlNavigator.WsdlNavigatorView;

  /**
   * Initialize all views and Controllers
   */
  public class OrchestraDesigner extends Box {

    /** Menu Bar */
    private var designerMenuBar:DesignerMenuBar;

    /** Tool Bar*/
    private var designerToolBar:DesignerToolBar;

    /** Attribute Navigator*/
    private var _propertyView:PropertyView;

    /** Process tab navigator */
    private var processTabNavigator:ProcessTabNavigator;

    /** bpmn palette */
    private var bpmnPalette:BpmnPalette;

    /** Tab Navigators */
    private var leftTabNavigator:TabNavigator;
    private var bottomTabNavigator:TabNavigator;


    /**  Microimage */
    private var microimage:Microimage;

    /** Divided boxes */
    private var mainHDividedBox:HDividedBox = new HDividedBox();
    private var leftVDividedBox:VDividedBox = new VDividedBox();
    private var middleVDividedBox:VDividedBox = new VDividedBox();

    /**
     * Initialize OrchestraDesigner.
     */
    public function OrchestraDesigner() {
      super();
      this.percentWidth = 100;
      this.percentHeight = 100;
      this.init();

    }

    /**
     * Initialize all views.
     */
    private function init():void {
      //initilize main menu
      designerMenuBar = new DesignerMenuBar();
      this.addChild(designerMenuBar);

      designerToolBar = new DesignerToolBar();
      this.addChild(designerToolBar);


      leftTabNavigator = new TabNavigator();
      leftTabNavigator.automationName = "leftTabNavigator";
      leftTabNavigator.autoLayout = true;
      leftTabNavigator.percentHeight = 100;
      leftTabNavigator.percentWidth = 100;

      var fileNavigatorTab:FileNavigatorTab = new FileNavigatorTab();
      leftTabNavigator.addChild(fileNavigatorTab);

      microimage = Microimage.getInstance();
      microimage.name = "microimage";


      // create process tab navigator
      processTabNavigator = new ProcessTabNavigator();
      processTabNavigator.automationName = "processTabNavigator";
      processTabNavigator.percentHeight = 75;
      processTabNavigator.percentWidth = 100;

      // create BPMN palette
      bpmnPalette = new BpmnPalette();
      bpmnPalette.automationName = "palette";
      bpmnPalette.percentHeight = 100;
      bpmnPalette.percentWidth = 100;
      leftTabNavigator.addChild(bpmnPalette);

      // create Wsdl service navigator
      var wsdlNavigator:WsdlNavigatorView = new WsdlNavigatorView();
      wsdlNavigator.automationName = "wsdlNavigatorView";
      wsdlNavigator.percentHeight = 100;
      wsdlNavigator.percentWidth = 100;
      leftTabNavigator.addChild(wsdlNavigator);

      leftTabNavigator.addEventListener(FlexEvent.CREATION_COMPLETE, leftTabComplete);


      //layout arrangement
      middleVDividedBox.name = "midVDBox";
      _propertyView = new PropertyView();
      _propertyView.automationName = "propertyView";
      _propertyView.percentHeight = 100;
      _propertyView.percentWidth = 100;

      bottomTabNavigator = new TabNavigator();
      bottomTabNavigator.name = "bottomTabNavigator";
      bottomTabNavigator.automationName = "propertyViewNavigator";
      bottomTabNavigator.autoLayout = true;
      bottomTabNavigator.percentWidth = 100;
      bottomTabNavigator.percentHeight = 25;

      middleVDividedBox.addChild(processTabNavigator);
      if (DesignerModelLocator.getInstance().isPropertiesViewVisible) {
        middleVDividedBox.addChild(bottomTabNavigator);
        bottomTabNavigator.addChild(_propertyView);
      }
      middleVDividedBox.percentHeight = 100;
      middleVDividedBox.percentWidth = 75;

      mainHDividedBox.name = "bottomHDBox";
      leftVDividedBox.name = "leftVDBox";
      leftVDividedBox.percentHeight = 100;
      leftVDividedBox.addChild(leftTabNavigator);


      microimage.percentHeight = 33;
      if (DesignerModelLocator.getInstance().isMicroImageViewVisible) {
        leftVDividedBox.addChild(microimage);
      }


      mainHDividedBox.addChild(leftVDividedBox);
      mainHDividedBox.addChild(middleVDividedBox);
      mainHDividedBox.percentHeight = 95;
      mainHDividedBox.percentWidth = 100;

      this.addChild(mainHDividedBox);
      this.setStyle("headerHeight", "20");
      this.setStyle("borderThicknessTop", "2");
      this.setStyle("borderThicknessLeft", "2");
      this.setStyle("borderThicknessRight", "2");
      this.setStyle("backgroundAlpha", "0.7");

      // change DividedBox style (thiner)
      mainHDividedBox.setStyle("horizontalGap", 3);
      leftVDividedBox.setStyle("verticalGap", 3);
      middleVDividedBox.setStyle("verticalGap", 3);

      mainHDividedBox.horizontalScrollPolicy = ScrollPolicy.OFF;
      mainHDividedBox.verticalScrollPolicy = ScrollPolicy.OFF;
      leftVDividedBox.horizontalScrollPolicy = ScrollPolicy.OFF;
      leftVDividedBox.verticalScrollPolicy = ScrollPolicy.OFF;
      middleVDividedBox.horizontalScrollPolicy = ScrollPolicy.OFF;
      middleVDividedBox.verticalScrollPolicy = ScrollPolicy.OFF;

      this.initToolTip();

      DesignerModelLocator.getInstance().addEventListener(PropertyChangeEvent.PROPERTY_CHANGE, this.onModelPropertyChanged);
    }

    private function initToolTip():void {
      var wiperight:WipeRight = new WipeRight();
      wiperight.duration = 600;
      wiperight.repeatCount = 1;
      ToolTipManager.showEffect = wiperight;
    }

    private function leftTabComplete(event:FlexEvent):void {
      var tab:Button;
      var idx:uint;
      var len:uint = leftTabNavigator.numChildren;
      for (idx = 0; idx < len; idx++) {
        tab = leftTabNavigator.getTabAt(idx) as Button;
        tab.automationName = "tab" + idx;
      }
    }

    private function onModelPropertyChanged(event:PropertyChangeEvent):void {
      if (event.kind == PropertyChangeEventKind.UPDATE) {
        if (event.property == "isPropertiesViewVisible") {

          if (DesignerModelLocator.getInstance().isPropertiesViewVisible) {
            bottomTabNavigator.addChild(_propertyView);
            if (bottomTabNavigator.parent == null) {
              middleVDividedBox.addChild(bottomTabNavigator);
            }
          } else {
            bottomTabNavigator.removeChild(_propertyView);
            if (bottomTabNavigator.numChildren == 0) {
              middleVDividedBox.removeChild(bottomTabNavigator);
            }
          }
        } else if (event.property == "isMicroImageViewVisible") {

          if (DesignerModelLocator.getInstance().isMicroImageViewVisible) {
            leftVDividedBox.addChild(microimage);
          } else {
            leftVDividedBox.removeChild(microimage);
          }
        }
      }
    }
  }
}

