/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.commands {
  import com.adobe.cairngorm.commands.ICommand;
  import com.adobe.cairngorm.control.CairngormEvent;

  import mx.collections.ArrayCollection;

  import org.ow2.orchestra.designer.bpmn.model.AbstractElementWithPosition;
  import org.ow2.orchestra.designer.bpmn.model.PoolModel;
  import org.ow2.orchestra.designer.bpmn.views.ProcessPoolView;
  import org.ow2.orchestra.designer.events.ProcessSelectionElementCoordinatesChangeEvent;

  public class MoveSelectedElementsCMD implements ICommand {

    public function MoveSelectedElementsCMD() {
    }

    public function execute(event:CairngormEvent):void {
      var processSelElCoordEvt:ProcessSelectionElementCoordinatesChangeEvent = event
              as ProcessSelectionElementCoordinatesChangeEvent;

      // save process model
      processSelElCoordEvt.processEditorModel.saveModelForUndo();

      var listSelectedElements:ArrayCollection = processSelElCoordEvt.processEditorModel.selectedElements;

      // Try to see if all positions (x,y) of the selected element can't become negative. In this case it disappear of the canvas
      // without scrollbar so do not permit this
      var i:int = 0;
      while (i < listSelectedElements.length) {
        if (listSelectedElements.getItemAt(i) is PoolModel) {
          // cannot move pools
          return;
        } else if (listSelectedElements.getItemAt(i) is AbstractElementWithPosition) {
          var elementModel:AbstractElementWithPosition = listSelectedElements.getItemAt(i) as AbstractElementWithPosition;
          if (((elementModel.x + processSelElCoordEvt.deltaX) < ProcessPoolView.POOL_HEADER_WIDTH)
                  || ((elementModel.y + processSelElCoordEvt.deltaY) < processSelElCoordEvt.processEditorModel.processModel.y)) {
            return; // can't move element do nothing on the model
          }
        }
        i++;
      }

      // If all selected element can't be visible after moving so change their coordinates effectively.
      i = 0;
      while (i < listSelectedElements.length) {
        if (listSelectedElements.getItemAt(i) is AbstractElementWithPosition) {
          var elementModel2:AbstractElementWithPosition = listSelectedElements.getItemAt(i) as AbstractElementWithPosition;
          elementModel2.x = elementModel2.x + processSelElCoordEvt.deltaX;
          elementModel2.y = elementModel2.y + processSelElCoordEvt.deltaY;
        }
        i++;
      }
    }
  }
}
