/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.model {
  import mx.collections.ArrayCollection;

  import org.ow2.orchestra.designer.bpmn.model.ProcessModel;
  import org.ow2.orchestra.designer.utils.DesignerUtil;
  import org.ow2.orchestra.designer.vo.FileNavigatorProcess;

  [Bindable]
  public class ProcessEditorModel {

    //Path of the file on the server to store the process
    private var _fileNavigatorProcess:FileNavigatorProcess;

    // Elements currently selected
    private var _selectedElements:ArrayCollection = new ArrayCollection();

    private var _processModel:ProcessModel;

    // For undo redo
    // Number of undo/redo elements to keep in memory
    private static const UNDO_REDO_QUEUE_LENGTH:int = 10;
    private var _undoState:ArrayCollection = new ArrayCollection();
    private var _redoState:ArrayCollection = new ArrayCollection();


    public function ProcessEditorModel() {
      _processModel = new ProcessModel();
    }


    public function get selectedElements():ArrayCollection {
      return _selectedElements;
    }

    public function set selectedElements(value:ArrayCollection):void {
      _selectedElements = value;
    }

    public function get processModel():ProcessModel {
      return _processModel;
    }

    public function set processModel(value:ProcessModel):void {
      _processModel = value;
      _selectedElements.removeAll();
    }

    public function get fileNavigatorProcess():FileNavigatorProcess {
      return DesignerModelLocator.getInstance().fileNavigatorModel.getProcess(_fileNavigatorProcess);
    }

    public function set fileNavigatorProcess(value:FileNavigatorProcess):void {
      _fileNavigatorProcess = value;
    }

    public function saveModelForUndo():void {
      _undoState.addItem(ProcessModel(DesignerUtil.copy(processModel)));
      if (_undoState.length > UNDO_REDO_QUEUE_LENGTH) {
        // undo max length is reached, remove the oldest process state
        _undoState.removeItemAt(0);
      }
      _redoState.removeAll();
    }

    public function undo():void {
      trace("Undo..");
      if (_undoState.length != 0) {
        _redoState.addItem(ProcessModel(DesignerUtil.copy(processModel)));
        processModel = ProcessModel(_undoState.removeItemAt(_undoState.length - 1));
        trace("Undo done");
      }
    }

    public function redo():void {
      if (_redoState.length != 0) {
        _undoState.addItem(ProcessModel(DesignerUtil.copy(processModel)));
        processModel = ProcessModel(_redoState.removeItemAt(_redoState.length - 1));
      }
    }
  }
}
