/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.view {
  import com.adobe.cairngorm.control.CairngormEventDispatcher;

  import flash.events.Event;

  import flexlib.containers.SuperTabNavigator;
  import flexlib.events.SuperTabEvent;

  import mx.binding.utils.BindingUtils;
  import mx.events.CollectionEvent;
  import mx.events.CollectionEventKind;
  import mx.events.IndexChangedEvent;

  import org.ow2.orchestra.designer.events.ProcessEvent;
  import org.ow2.orchestra.designer.model.DesignerModelLocator;
  import org.ow2.orchestra.designer.model.ProcessEditorModel;
  import org.ow2.orchestra.designer.view.processEditor.ProcessEditorView;

  public class ProcessTabNavigator extends SuperTabNavigator {

    private var modelLocator:DesignerModelLocator = DesignerModelLocator.getInstance();

    public function ProcessTabNavigator() {
      super();
      //view init

      this.dragEnabled = false; //disable drag function

      this.percentHeight = 100;
      this.percentWidth = 100;
      this.popUpButtonPolicy = "off";

      // listen to collection changes
      modelLocator.openedProcesses.addEventListener(CollectionEvent.COLLECTION_CHANGE, processesChanged);

      this.addEventListener(SuperTabEvent.TAB_CLOSE, closeProcessEditorView);

      // Active the tab matching to the selected process
      BindingUtils.bindSetter(setSelectedProcessEditor, modelLocator, "selectedProcessEditor");

      // Change model selected process when a tab is selected
      this.addEventListener(IndexChangedEvent.CHANGE, selectProcessEditorView);
    }


    private function processesChanged(collectionEvent:CollectionEvent):void {

      // event instance of CollectionEvent
      // check kind attribute for "add, delete, move..."
      // get added or removed child

      // changed processes.
      var changedProcesses:Array = collectionEvent.items;

      if (collectionEvent.kind == CollectionEventKind.ADD) {
        // for each added process model, add a new process view in a new tab
        for each (var addedProcessEditor:ProcessEditorModel in changedProcesses) {
          // create process view
          var addedProcessEditorView:ProcessEditorView = new ProcessEditorView(addedProcessEditor);

          this.addChild(addedProcessEditorView);
          addedProcessEditorView.percentHeight = 100;
          addedProcessEditorView.percentWidth = 100;
        }
      }
      else if (collectionEvent.kind == CollectionEventKind.REMOVE) {
        // for each removed process model,
        // find the child process view for this model, and remove it.
        for each (var removedProcessEditor:ProcessEditorModel in changedProcesses) {
          var removedProcessEditorView:ProcessEditorView = getProcessEditorView(removedProcessEditor);
          if (removedProcessEditorView != null) {
            this.removeChild(removedProcessEditorView);
          }
        }
      }
      else if (collectionEvent.kind == CollectionEventKind.RESET) {
        // for each removed process model,
        // find the child process view for this model, and remove it.
        for each (var processEditorViewToRemove:ProcessEditorView in this.getChildren()) {
          this.removeChild(processEditorViewToRemove);
        }
      }
    }

    /**
     * Active the tab matching to the selected process
     */
    private function setSelectedProcessEditor(selectedProcessEditor:ProcessEditorModel):void {
      var processEditorView:ProcessEditorView = getProcessEditorView(selectedProcessEditor);
      if (processEditorView != null) {
        this.selectedChild = processEditorView;
      }
    }

    /**
     * Return the process editor view associated to the process editor model
     */
    private function getProcessEditorView(processEditorModel:ProcessEditorModel):ProcessEditorView {
      var result:ProcessEditorView = null;
      for each (var child:ProcessEditorView in this.getChildren()) {
        if (child.processEditorModel == processEditorModel) {
          result = child;
          break;
        }
      }
      return result;
    }


    /**
     * Active the tab matching to the selected process
     */
    public function closeProcessEditorView(event:SuperTabEvent):void {
      event.preventDefault();

      //get the processView selected to be closed
      var processEditorView:ProcessEditorView = this.getChildAt(event.tabIndex) as ProcessEditorView;

      // Dispatch an event to close the process
      var evt:ProcessEvent = new ProcessEvent(ProcessEvent.PROCESS_CLOSE, processEditorView.processEditorModel);
      CairngormEventDispatcher.getInstance().dispatchEvent(evt);

    }

    /**
     * Select the process matching the selected tab
     */
    public function selectProcessEditorView(event:IndexChangedEvent):void {
      //get the processView selected to be closed
      var processEditorView:ProcessEditorView = this.getChildAt(event.newIndex) as ProcessEditorView;

      // Dispatch an event to close the process
      var evt:ProcessEvent = new ProcessEvent(ProcessEvent.PROCESS_SELECT, processEditorView.processEditorModel);
      CairngormEventDispatcher.getInstance().dispatchEvent(evt);

    }

  }
}
