/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


package org.ow2.orchestra.designer.view.processEditor {
  import flash.display.DisplayObject;
  import flash.events.MouseEvent;
  import flash.geom.Point;

  import mx.states.SetEventHandler;
  import mx.states.State;

  import org.ow2.orchestra.designer.events.ProcessSelectionElementCoordinatesChangeEvent;

  public class ProcessEditorMovingStateHelper extends ProcessEditorFlowStateHelper {

    public function ProcessEditorMovingStateHelper(processEditorView:ProcessEditorView) {
      super(processEditorView);
    }

    /**
     * Generate "moving" state.
     * In the moving state:
     * - on mouse move, the selected elements move to follow the mouse cursor.
     * - on mouse up, this view goes back to the "default" state.
     * - when the mouse leaves this view, the view goes back to the "default" state.
     */
    public function addMovingState():void {
      var state:State = new State();
      state.name = "moving";
      processEditorView.states.push(state);

      // handler for mouse up event
      var mouseUpListener:SetEventHandler = new SetEventHandler(processEditorView, MouseEvent.MOUSE_UP);
      mouseUpListener.handlerFunction = movingMouseUp;
      state.overrides.push(mouseUpListener);

      // handler for mouse move event
      var mouseMoveListener:SetEventHandler = new SetEventHandler(processEditorView, MouseEvent.MOUSE_MOVE);
      mouseMoveListener.handlerFunction = movingMouseMove;
      state.overrides.push(mouseMoveListener);
    }

    /**
     * handler for mouse up events when this view is in the "moving" state.
     * The view goes back to the "default" state.
     */
    private function movingMouseUp(event:MouseEvent):void {
      // go back to the "default" state.
      processEditorView.currentState = "default";
    }


    /**
     * handler for mouse move event when this view is in the "moving" state.
     * Update selected elements positions to follow the mouse.
     */
    private function movingMouseMove(event:MouseEvent):void {
      // hide context panel
      processEditorView.hideElementContextPanel();

      // Get mouse event position. The coordinates are relative to the target element
      var targetLocalMousePoint:Point = new Point(event.localX, event.localY);
      // Transform the coordinates to be relative to this view
      var mousePoint:Point = processEditorView.globalToLocal(DisplayObject(event.target).localToGlobal(targetLocalMousePoint));

      var deltaX:int = mousePoint.x - processEditorView.selectionRect.x;
      var deltaY:int = mousePoint.y - processEditorView.selectionRect.y;
      processEditorView.selectionRect.x = mousePoint.x;
      processEditorView.selectionRect.y = mousePoint.y;

      // selected element is moved, generate a process modification event.
      new ProcessSelectionElementCoordinatesChangeEvent(
              ProcessSelectionElementCoordinatesChangeEvent.COORDINATES_CHANGE,
              processEditorView.processEditorModel,
              deltaX,
              deltaY
              ).dispatch();
    }
  }
}
