/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

package org.ow2.orchestra.designer.view.properties {
  import mx.collections.ArrayCollection;
  import mx.controls.ComboBox;
  import mx.controls.Label;
  import mx.core.UIComponent;
  import mx.events.CollectionEvent;
  import mx.events.ListEvent;

  import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
  import org.ow2.orchestra.designer.bpmn.model.PoolModel;
  import org.ow2.orchestra.designer.bpmn.model.operations.InterfaceModel;
  import org.ow2.orchestra.designer.bpmn.model.operations.OperationModel;
  import org.ow2.orchestra.designer.events.ElementPropertyChangeEvent;
  import org.ow2.orchestra.designer.model.DesignerModelLocator;

  [Bindable]
  public class OperationProperty extends AbstractProperty {
    private var _itemEditor:ComboBox;
    private var _itemRenderer:Label;

    private var defaultOperation:OperationModel = new OperationModel();

    private var _isExternalOperation:Boolean;

    public function OperationProperty(propertyName:String, description:String, value:OperationModel, isExternalOperation:Boolean = false) {
      super(propertyName, description);
      _isExternalOperation = isExternalOperation;

      _itemRenderer = new Label();
      _itemEditor = new ComboBox();
      _itemEditor.labelFunction = labelOperation;
      _itemEditor.automationName = propertyName + "Editor";

      // get list of operations
      getOperationList();

      if (value != null) {
        _itemRenderer.text = labelOperation(value as OperationModel);
        _itemEditor.selectedItem = value;
      } else {
        _itemRenderer.text = " ";
        _itemEditor.selectedItem = defaultOperation;
      }
      _itemEditor.addEventListener(ListEvent.CHANGE, this.valueChanged, false, 0, true);
    }

    public override function get itemEditor():UIComponent {
      return _itemEditor;
    }

    public override function get itemRenderer():UIComponent {
      return _itemRenderer;
    }

    private function valueChanged(event:ListEvent):void {
      var element:AbstractElement = getSelectedElement();
      if (element == null) {
        return;
      }
      var selectedOperation:OperationModel = _itemEditor.selectedItem as OperationModel;
      if (selectedOperation == defaultOperation) {
        selectedOperation = null;
      }
      new ElementPropertyChangeEvent(ElementPropertyChangeEvent.PROPERTY_CHANGED, element, _propertyName,
              selectedOperation).dispatch();
    }

    private function labelOperation(value:OperationModel):String {
      if (value == null || value.interfaceModel == null) {
        return " ";
      }
      return value.name + " (" + value.interfaceModel.name + ")";
    }

    private function getOperationList(event:CollectionEvent = null):void {
      // fill possible values
      var values:ArrayCollection = new ArrayCollection();
      //Add a first null item.
      values.addItem(defaultOperation);
      if (!_isExternalOperation) {
        for each (var processInterface:InterfaceModel in
                DesignerModelLocator.getInstance().selectedProcessEditor.processModel.interfaces.values) {
          for each (var op:OperationModel in processInterface.operations.values) {
            values.addItem(op);
          }
        }
      } else {
        for each (var e:Object in DesignerModelLocator.getInstance().selectedProcessEditor.processModel.elements) {
          if (e is PoolModel) {
            for each (var processInterface:InterfaceModel in (e as PoolModel).interfaces.values) {
              for each (var op:OperationModel in processInterface.operations.values) {
                values.addItem(op);
              }
            }
          }
        }
      }

      _itemEditor.dataProvider = values;
    }
  }
}
